@@ -1,11 +1,19 @@
 #pragma once
 
 #if __cpp_lib_string_view >= 201603L
-#  include <string_view>
-#else
-#  include <panda/lib/hash.h>
-#  include <panda/basic_string_view.h>
+#   define PANDA_LIB_USE_PANDA_LIB_STRING_VIEW 1
+// HACK! Clang contains <string_view> and includes it from <string>, but it does not define __cpp_lib_string_view 
+#elif  __clang__ && defined(__has_include)
+#   if __has_include(<string_view>)
+#       define PANDA_LIB_USE_PANDA_LIB_STRING_VIEW 1
+#   endif
+#endif
 
+#if defined(PANDA_LIB_USE_PANDA_LIB_STRING_VIEW)
+#       include <string_view>
+#else
+#       include <panda/lib/hash.h>
+#       include <panda/basic_string_view.h>
 namespace std {
 
     typedef basic_string_view<char>     string_view;
@@ -41,7 +49,7 @@ namespace std {
         }
     };
 
-    namespace literals { namespace string_view_literals {
+    inline namespace literals { namespace string_view_literals {
 
         // uncomment when -Wno-literal-suffix works
         //constexpr string_view    operator "" sv (const char*     str, size_t len) noexcept { return string_view(str, len); }
@@ -51,5 +59,5 @@ namespace std {
 
     }}
 }
-
+#undef PANDA_LIB_USE_PANDA_LIB_STRING_VIEW 
 #endif
